<?php

namespace Drupal\datafield\Controller;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Class Json data Controller.
 *
 * @package Drupal\datafield\Controller
 */
class JsonController extends ControllerBase {

  /**
   * Cache service.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected CacheBackendInterface $cache;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected Connection $database;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected EntityFieldManagerInterface $entityFieldManager;

  /**
   * The language manager service.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * Constructor Json search.
   */
  public function __construct(CacheBackendInterface $staticCache, Connection $database, EntityTypeManagerInterface $entity_type_manager, EntityFieldManagerInterface $entity_field_manager, LanguageManagerInterface $language_manager) {
    $this->cache = $staticCache;
    $this->database = $database;
    $this->entityTypeManager = $entity_type_manager;
    $this->languageManager = $language_manager;
    $this->entityFieldManager = $entity_field_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('cache.default'),
      $container->get('database'),
      $container->get('entity_type.manager'),
      $container->get('entity_field.manager'),
      $container->get('language_manager'),
    );
  }

  /**
   * Get taxonomies with parents.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request service.
   * @param string $entity_type
   *   The entity type service.
   * @param string $bundle
   *   Bundle.
   * @param string $field_name
   *   Field name.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   Ajax to get select list.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function json(Request $request, $entity_type, $bundle, $field_name): JsonResponse {
    $data = [];
    $storage = $this->entityTypeManager->getStorage($entity_type);
    [$tableName] = $storage->getTableMapping()
      ->getAllFieldTableNames($field_name);
    $q = $request->query->get('q');
    $machine_name = $request->query->get('machine_name');
    $fieldQuery = $field_name . '_' . $machine_name;
    $bundle_fields = $this->entityFieldManager->getFieldDefinitions($entity_type, $bundle);
    $field_definition = $bundle_fields[$field_name];
    $fields = array_keys($field_definition->getSetting('columns'));
    $fieldNames = [];
    if (!empty($fields)) {
      foreach ($fields as $fieldName) {
        $fieldNames[$fieldName] = $field_name . '_' . $fieldName;
      }
    }
    $language = $this->languageManager->getCurrentLanguage()->getId();
    $query = $this->database->select($tableName, 'f');
    $query->fields('f');
    $query->condition('f.bundle', $bundle);
    $query->condition('f.langcode', $language);
    if (!empty($machine_name) && !empty($q)) {
      $query->distinct(TRUE);
      $query->condition('f.' . $fieldQuery, '%' . $q . '%', 'LIKE');
    }
    $limit = $request->query->get('limit');
    if (!empty($limit) && is_numeric($limit)) {
      $query->range(0, $limit);
    }
    $entity_id = $request->query->get('entity_id');
    if (!empty($entity_id)) {
      $query->condition('f.entity_id', $entity_id);
    }
    $result = $query->execute();
    foreach ($result as $row) {
      $values = [];
      foreach ($fields as $fieldName) {
        $values[$fieldName] = $row->{$field_name . '_' . $fieldName};
      }
      if (!empty($machine_name) && !empty($q)) {
        $label = $row->$fieldQuery;
        $value = $row->$fieldQuery;
        $fill = $request->query->get('fill');
        $delta = $request->query->get('delta');
        if ($fill) {
          $label = implode(' | ', $values);
        }
        $data[] = [
          'value' => $value,
          'label' => $label,
          'data' => $values,
          'field' => $field_name,
          'delta' => $delta,
          'name' => $machine_name,
        ];
      }
      else {
        $values['delta'] = $row->delta;
        if (empty($entity_id)) {
          $values['entity_id'] = $row->entity_id;
        }
        $data[] = $values;
      }
    }
    return new JsonResponse($data);
  }

}
